<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\SetupRequest;
use App\Utils\ConsoleUtils;
use Doctrine\DBAL\Driver\PDOException;
use Jackiedo\DotenvEditor\Facades\DotenvEditor;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\{Artisan, DB};
use Illuminate\Support\Str;
use PDO;
use Symfony\Component\Process\Process;

class SetupController extends Controller
{
    public function __invoke(SetupRequest $request)
    {
        $validatedData = $request->validated();

        $password = Str::random(20);

        $configurations = (object) [
            'root_user' => $validatedData['db_username'], 'root_pwd' => $validatedData['db_password'],
            'dialer_pwd' => $password
        ];
        $configurations = json_encode($configurations);

        ConsoleUtils::execute('dialer', "install {$configurations}", $result);

        //test database connections
        config([
            'database.connections.asterisk.host' => '127.0.0.1',
            'database.connections.asterisk.port' => '3306',
            'database.connections.asterisk.database' => 'asterisk',
            'database.connections.asterisk.username' => 'dialer',
            'database.connections.asterisk.password' => $password,
        ]);

        try {
            DB::connection('asterisk')->getPdo();
        } catch (QueryException | PDOException $e) {
            return response()->json(['message' => 'Asterisk database connection error'], 500);
        }

        config([
            'database.connections.mysql.host' => '127.0.0.1',
            'database.connections.mysql.port' => '3306',
            'database.connections.mysql.database' => 'sonata_dialer',
            'database.connections.mysql.username' => 'dialer',
            'database.connections.mysql.password' => $password,
        ]);

        try {
            Artisan::call('logs:clear');
            Artisan::call('migrate --force');
        } catch (QueryException | PDOException $e) {
            return response()->json(['message' => 'Database connection error'], 500);
        }

        $editor = DotenvEditor::load();

        $editor->setKeys([
            'DB_PASSWORD' => $password,
            'DB_ASTERISK_PASSWORD' => $password
        ]);

        $editor->save();

        Artisan::call('config:cache');

        // Create a new process to run the systemctl restart command
        $process = new Process(['systemctl', 'restart', 'sonata-dialer']);
        $process->run();

        $process = new Process(['systemctl', 'restart', 'sonata-dialer-worker@*']);
        $process->run();

        return ['message' => __('finished_setup')];
    }
}
